<?php
/**
 * Created by PhpStorm.
 * User: eugeniofonseca
 */

namespace Service\Integration;


use Util\Messages;
use Zeedhi\Framework\DTO\Request;
use Util\InstanceProvider;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Util\Utilities;

class IntegrationSM {

    const CREATE_PROPOSAL = "/IntegracaoPm/salvarProposta";
    const ACCEPTED_PROPOSAL = "/IntegracaoPm/aceitarProposta";
    const CANCEL_PROPOSAL = "/IntegracaoPm/cancelarProposta";
    const LINK_DOCUMENT_PROPOSAL = "/IntegracaoPm/gerarDocumentosProposta";
    const APROVAR_CONTRATO = "/IntegracaoPm/aprovarContrato";

    private static function getDataRequestDataSet($dataSet) {
        $dataSet = array(
            "requestType" => Request::TYPE_DATA_SET,
            "dataset" => $dataSet
        );

        return $dataSet;
    }

    public static function validateProposal($proposalValues,$initialSave = true)
    {

        $validateError = array();
        if (empty($proposalValues['REPORTER'])) {
            $validateError[] = Messages::ERR_BUDGET_REPORTER;
        }
        if($initialSave){
            if(empty($proposalValues['PRODUCT_NAME'])){
                $validateError[] = Messages::ERR_BUDGET_INTEGRATION_DEVOPS;
            }else if (empty($proposalValues['OM_PRODUTO'])) {
                $validateError[] = Messages::ERR_BUDGET_INTEGRATION_PRODUCT . $proposalValues['PRODUCT_NAME'] . Messages::ERR_BUDGET_INTEGRATION_2;
            }
            if (empty($proposalValues['BUDGET_ID'])) {
                $validateError[] = Messages::ERR_BUDGET_NOT_CREATED;

            }
            if (empty($proposalValues['SPONSOR_NAME'])) {
                $validateError[] = Messages::ERR_BUDGET_INTEGRATION_SPONSOR_2;
            }else if (empty($proposalValues['SPONSOR'])) {
                $validateError[] = Messages::ERR_BUDGET_INTEGRATION_SPONSOR . $proposalValues['SPONSOR_NAME'] . Messages::ERR_BUDGET_INTEGRATION_2;
            }
        }

        if($validateError){
            $message = Messages::ERR_INTEGRATION_SM_ITEMS;
            foreach ($validateError as $error) {
                $message .= "<BR/>".$error;
            }
            throw new \Exception($message,1);
        }
    }

    private  static function getDataRequestRow($row) {
        $dataRow = array(
            "requestType" => Request::TYPE_ROW,
            "row" => $row
        );

        return $dataRow;
    }

    public static function validateSpecification($issueId)
    {
        $budgetRepository = InstanceProvider::getRepository()->getBudget();
        $proposalValues = $budgetRepository->getProposalValues($issueId);
        self::validateProposal($proposalValues);

    }

    public static function integrationSMApproveContract($issueId, $organizationId)
    {
        try {
            $smRow = self::getRowIntegration($issueId, $organizationId);
            if(!empty($smRow['id'])){
                $smDataSet = self::getDataRequestDataSet(array($smRow));
                $token = LibTokenIntegration::requestToken($smRow['urlSm']);
                self::requestSMRoute(self::APROVAR_CONTRATO, $token, $smDataSet, $smRow['urlSm']);
                return true;
            }else{
                return false;
            }

        } catch (\Exception $e) {
            throw new \Exception(Messages::ERR_INTEGRATION_SM. $e->getMessage(), $e->getCode());
        }
    }

    public static function integrationSMFinishSpecification($issueId, $organizationId)
    {
        try {
            $smRow = self::getRowIntegrationCreateProposal($issueId, $organizationId);
            if($smRow){
                $smDataSet = self::getDataRequestDataSet(array($smRow));
                $token = LibTokenIntegration::requestToken($smRow['urlSm']);
                $proposal = self::requestSMRoute(self::CREATE_PROPOSAL, $token, $smDataSet, $smRow['urlSm']);
                if(empty($proposal)){
                    throw new \Exception(Messages::ERR_BUDGET_INTEGRATION_SM);
                }
                return $proposal['proposta']['id'];
            }
        } catch (\Exception $e) {
            throw new \Exception(Messages::ERR_INTEGRATION_SM. $e->getMessage(), $e->getCode());
        }
    }

    public static function integrationSMAccept($issueId, $organizationId)
    {
        try {
            $smRow = self::getRowIntegration($issueId, $organizationId);
            if ($smRow['id']) {
                $smDataSet = self::getDataRequestDataSet(array($smRow));
                $token = LibTokenIntegration::requestToken($smRow['urlSm']);
                self::requestSMRoute(self::ACCEPTED_PROPOSAL, $token, $smDataSet, $smRow['urlSm']);
            }
        } catch (\Exception $e) {
            throw new \Exception(Messages::ERR_INTEGRATION_SM . $e->getMessage(), $e->getCode());
        }
    }

    public static function linkDownloadBudgetSm($issueId, $organizationId)
    {
        $linkDownloadBudgetSm = array();
        try {
            $smRow = self::getRowIntegration($issueId, $organizationId);
            if ($smRow['id']) {
                $smDataSet = self::getDataRequestDataSet(array($smRow));
                $token = LibTokenIntegration::requestToken($smRow['urlSm']);
                $linkDownloadBudgetSm = self::requestSMRoute(self::LINK_DOCUMENT_PROPOSAL, $token, $smDataSet, $smRow['urlSm']);
                if(isset($linkDownloadBudgetSm['documentos']) && isset($linkDownloadBudgetSm['documentos']['links']) ){
                    $linkDownloadBudgetSm = $linkDownloadBudgetSm['documentos']['links'];
                }
            }
        } catch (\Exception $e) {
            throw new \Exception(Messages::ERR_INTEGRATION_SM . $e->getMessage(), $e->getCode());
        }
        return $linkDownloadBudgetSm;
    }

    public static function integrationSMCancel($issueId, $organizationId)
    {
        try {
            $smRow = self::getRowIntegration($issueId, $organizationId);
            if ($smRow['id']) {
                $smDataSet = self::getDataRequestDataSet(array($smRow));
                $token = LibTokenIntegration::requestToken($smRow['urlSm']);
                self::requestSMRoute(self::CANCEL_PROPOSAL, $token, $smDataSet, $smRow['urlSm']);
            }
        } catch (\Exception $e) {
            throw new \Exception(Messages::ERR_INTEGRATION_SM . $e->getMessage(), $e->getCode());
        }
    }

    public static function requestSMRoute($route,$token, $data = array(),$url){

        $data = json_encode($data);
        $url = $url.$route;

        $header = array();
        $header[] = 'OAuth-Token: '.$token;
        $header[] = 'OAuth-SecretId: '.LibTokenIntegration::CLIENT_SECRET;
        $header[] = 'OAuth-KeepConnected: Yes';
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: '.strlen($data);
        $header[] = 'Expect:';

        $curl_session = curl_init();

        curl_setopt($curl_session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl_session, CURLOPT_URL, $url);
        curl_setopt($curl_session, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl_session, CURLOPT_TIMEOUT, 86400);
        curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, true);
        $result = curl_exec($curl_session);
        $error = curl_error($curl_session);
        curl_close($curl_session);
        $result = json_decode($result, true);
        if (!empty($result['error'])) {
            throw new \Exception($result['error']);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else if (!empty($result['messages'])) {
            throw new \Exception($result['messages'][0]['message']);
        }  else {
            $result = isset($result['dataset']) ? $result['dataset'] : null;
        }

        return $result;
    }

    public static function getBudgetPlotsIntegration($issueId){
        $budgetRepository = InstanceProvider::getRepository()->getBudget();
        $budgetPlots = $budgetRepository->findIssueBudgetPlots($issueId);
        $parcelIntegration = array();
        foreach ($budgetPlots as $parcel) {
            $parcelIntegration[] = array(
                "value" => floatval($parcel['VALUE_PLOTS']),
                "billingDays" =>  intval($parcel['DAYS_PAYMENT'])
            );
        }
        return $parcelIntegration;
    }

    public static function getRowIntegrationCreateProposal($issueId,$organizationId)
    {
        $budgetRepository = InstanceProvider::getRepository()->getBudget();
        $baseUri = Utilities::getBaseUri();
        $cdOperCurrent = \Util\Utilities::getEnvironment()->getCurrentCdoperador();
        $proposalValues = $budgetRepository->getProposalValues($issueId);
        $proposalItemValues = $budgetRepository->getProposalItemValues($issueId);
        $budgetPlots = self::getBudgetPlotsIntegration($issueId);
        $urlSm = $budgetRepository->getUrlIntegrationSM($organizationId,$baseUri);
        self::validateProposal($proposalValues);
        $maintenanceDiscounts = $budgetRepository->findManutIssueDiscount($issueId);
        $maintenanceInfo = null;
        if($maintenanceDiscounts){
            $maintenanceInfo = array();
            $maintenance = $budgetRepository->MaintenanceCost($organizationId);
            $budgetPrice = $budgetRepository->findManutIssueBudget($issueId);
            $finalPrice = round($budgetPrice[0]["TOTAL_VALUE"] - ($budgetPrice[0]["TOTAL_VALUE"] * $budgetPrice[1]["TOTAL_VALUE"] / 100), 2);
            $maintenancePrice = round($finalPrice * $maintenance / 100, 2);
            $maintenancePrice = number_format($maintenancePrice, 2, '.', '');
            $maintenanceInfo['originalMaintenance'] = $maintenancePrice;
            $maintenanceInfo['maintenanceDisc'] = $maintenanceDiscounts[0]['MAINTENANCE_VALUE'];
            $maintenanceInfo['percentualValue'] = $maintenanceDiscounts[0]['MAINTENANCE_PERCENT'];
            $maintenanceInfo['maintenanceValue'] = $maintenancePrice - $maintenanceDiscounts[0]['MAINTENANCE_VALUE'];
        }
        $smRow = array(
            "urlSm" => $urlSm,
            "requester" => array(
                "userId" => $cdOperCurrent ? $cdOperCurrent : $proposalValues['REPORTER'],
                "organizationId" => $proposalValues['ORGANIZATION'],
            ),
            "sponsor" => $proposalValues['SPONSOR'],
            "issueId" => $issueId,
            "product" => $proposalValues['OM_PRODUTO'],
            "description" => $proposalValues['DESCRIPTION'],
            "items" => $proposalItemValues,
            "instalments" => $budgetPlots,
            "maintenance" => $maintenanceInfo,
            "interlocutor" =>
                array(
                    "id" => $proposalValues['REPORTER'],
                    "name" => $proposalValues['REPORTER_NAME'],
                    "email" => $proposalValues['REPORTER_EMAIL']
                ),

            "discount" => array(
                "percent" => $proposalValues['DISCOUNT_PERCENT'],
                "value" => $proposalValues['DISCOUNT_VALUE']
            )
        );
        return $smRow;
    }

    public static function getRowIntegration($issueId,$organizationId)
    {
        $budgetRepository = InstanceProvider::getRepository()->getBudget();
        $baseUri = Utilities::getBaseUri();
        /** @var Entities\Allocation $reporter */
        $urlSm = $budgetRepository->getUrlIntegrationSM($organizationId,$baseUri);
        if($urlSm != $baseUri){
            $proposalValues = $budgetRepository->getProposalValues($issueId);
            self::validateProposal($proposalValues,false);
            $cdOperCurrent = \Util\Utilities::getEnvironment()->getCurrentCdoperador();
            $smRow = array(
                "urlSm" => $urlSm,
                "requester" => array(
                    "userId" => $cdOperCurrent ? $cdOperCurrent : $proposalValues['REPORTER'],
                    "organizationId" => $proposalValues['ORGANIZATION'],
                ),
                "id" => $proposalValues['EXTERNAL_ID'],

            );
        }else{
            $smRow = array(
                "id" => null,
            );
        }
        return $smRow;
    }

}
